package jcatest.outboundtcp;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.net.Socket;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

import javax.resource.ResourceException;
import javax.resource.spi.ConnectionEvent;
import javax.resource.spi.ConnectionEventListener;
import javax.resource.spi.ConnectionRequestInfo;
import javax.resource.spi.LocalTransaction;
import javax.resource.spi.ManagedConnection;
import javax.resource.spi.ManagedConnectionMetaData;
import javax.security.auth.Subject;
import javax.transaction.xa.XAResource;

public class OutboundTCPManagedConnection implements ManagedConnection {
	private List<ConnectionEventListener> cels;
	private OutboundTCPConnection con;
	private boolean debug;
	private Socket sck;
	private OutputStream os;
	private InputStream is;
	private PrintWriter lw;
	public OutboundTCPManagedConnection(String host, int port, boolean debug) throws ResourceException {
		cels = new ArrayList<ConnectionEventListener>();
		con = null;
		this.debug = debug;
		try {
			sck = new Socket(host, port);
			os = sck.getOutputStream();
			is = sck.getInputStream();
		} catch (UnknownHostException e) {
			e.printStackTrace();
			error();
			throw new ResourceException("No connection");
		} catch (IOException e) {
			e.printStackTrace();
			error();
			throw new ResourceException("No connection");
		}
	}
	public void deallocate() {
		if(debug) {
			System.out.println("OutboundTCPManagedConnection deallocate");
		}
		for(ConnectionEventListener cel : cels) {
			ConnectionEvent ce = new ConnectionEvent(this, ConnectionEvent.CONNECTION_CLOSED);
			ce.setConnectionHandle(con);
			cel.connectionClosed(ce);
		}
	}
	public byte[] execute(byte[] req) {
		try {
			os.write(req);
			os.flush();
			if(debug) {
				System.out.println(req.length + " bytes write");
			}
			byte[] resp = new byte[req.length];
			int n = 0;
			while(n < resp.length) {
				n += is.read(resp, n, resp.length - n);
			}
			if(debug) {
				System.out.println(resp.length + " bytes read");
			}
			return resp;
		} catch (IOException e) {
			e.printStackTrace();
			error();
			return null;
		}
	}
	private void error() {
		if(debug) {
			System.out.println("OutboundTCPManagedConnection error");
		}
		for(ConnectionEventListener cel : cels) {
			ConnectionEvent ce = new ConnectionEvent(this, ConnectionEvent.CONNECTION_ERROR_OCCURRED);
			ce.setConnectionHandle(con);
			cel.connectionErrorOccurred(ce);
		}
	}
	@Override
	public void cleanup() throws ResourceException {
		if(debug) {
			System.out.println("OutboundTCPManagedConnection cleanup");
		}
	}
	@Override
	public void destroy() throws ResourceException {
		if(debug) {
			System.out.println("OutboundTCPManagedConnection destroy");
		}
		try {
			if(os != null) {
				os.close();
			}
			if(is != null) {
				is.close();
			}
			if(sck != null) {
				sck.close();
			}
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	@Override
	public OutboundTCPConnection getConnection(Subject sub, ConnectionRequestInfo cri) throws ResourceException {
		if(debug) {
			System.out.println("OutboundTCPManagedConnection created an OutboundTCPConnectionImpl");
		}
		con = new OutboundTCPConnectionImpl(this, debug);
		return con;
	}
	@Override
	public LocalTransaction getLocalTransaction() throws ResourceException {
		throw new ResourceException("Transaction not supported");
	}
	@Override
	public ManagedConnectionMetaData getMetaData() throws ResourceException {
		throw new ResourceException("MetaData not supported");
	}
	@Override
	public XAResource getXAResource() throws ResourceException {
		throw new ResourceException("XAResource not supported");
	}
	@Override
	public void associateConnection(Object con) throws ResourceException {
		this.con = (OutboundTCPConnection)con;
	}
	@Override
	public PrintWriter getLogWriter() throws ResourceException {
		return lw;
	}
	@Override
	public void setLogWriter(PrintWriter lw) throws ResourceException {
		this.lw = lw;
	}
	@Override
	public void addConnectionEventListener(ConnectionEventListener cel) {
		cels.add(cel);
	}
	@Override
	public void removeConnectionEventListener(ConnectionEventListener cel) {
		cels.remove(cel);
	}
}
